@@ -108,6 +108,8 @@ struct _GrEditPage
         GtkWidget *timer_title;
         GtkWidget *preview_stack;
 
+        GtkWidget *error_field;
+
         GrRecipeSearch *search;
 
         GtkSizeGroup *group;
@@ -151,6 +153,14 @@ dismiss_error (GrEditPage *page)
         gtk_revealer_set_reveal_child (GTK_REVEALER (page->error_revealer), FALSE);
 }
 
+static void
+focus_error (GrEditPage *page)
+{
+        dismiss_error (page);
+        if (page->error_field)
+                gtk_widget_grab_focus (page->error_field);
+}
+
 static void add_image_cb (GrEditPage *page);
 static void set_unsaved  (GrEditPage *page);
 static void add_list     (GrEditPage *page);
@@ -454,8 +464,10 @@ populate_season_combo (GrEditPage *page)
         }
 }
 
-static char *
-collect_ingredients (GrEditPage *page)
+static gboolean
+collect_ingredients (GrEditPage  *page,
+                     GtkWidget  **error_field,
+                     char       **ingredients)
 {
         GString *s;
         GList *children, *l;
@@ -467,6 +479,13 @@ collect_ingredients (GrEditPage *page)
                 GtkWidget *list = l->data;
                 g_autofree char *segment = NULL;
 
+                *error_field = gr_ingredients_viewer_has_error (GR_INGREDIENTS_VIEWER (list));
+                if (*error_field) {
+                        *ingredients = NULL;
+                        g_string_free (s, TRUE);
+                        return FALSE;
+                }
+
                 g_object_get (list, "ingredients", &segment, NULL);
                 if (s->len > 0)
                         g_string_append (s, "\n");
@@ -474,7 +493,10 @@ collect_ingredients (GrEditPage *page)
         }
         g_list_free (children);
 
-        return g_string_free (s, FALSE);
+        *error_field = NULL;
+        *ingredients = g_string_free (s, FALSE);
+
+        return TRUE;
 }
 
 static void
@@ -884,6 +906,12 @@ gr_edit_page_init (GrEditPage *page)
         gtk_widget_set_has_window (GTK_WIDGET (page), FALSE);
         gtk_widget_init_template (GTK_WIDGET (page));
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+        // Without this, we get border artifacts left behind when adding and removing
+        // ingredients :-(
+        gtk_container_set_reallocate_redraws (GTK_CONTAINER (page->ingredients_box), TRUE);
+G_GNUC_END_IGNORE_DEPRECATIONS
+
         page->group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
         populate_cuisine_combo (page);
@@ -1087,6 +1115,7 @@ gr_edit_page_class_init (GrEditPageClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrEditPage, cooking_view);
 
         gtk_widget_class_bind_template_callback (widget_class, dismiss_error);
+        gtk_widget_class_bind_template_callback (widget_class, focus_error);
         gtk_widget_class_bind_template_callback (widget_class, add_image_cb);
         gtk_widget_class_bind_template_callback (widget_class, remove_image_cb);
         gtk_widget_class_bind_template_callback (widget_class, rotate_image_left_cb);
@@ -1493,9 +1522,19 @@ gr_edit_page_save (GrEditPage *page)
         GrDiets diets;
         GPtrArray *images;
 
+        page->error_field = NULL;
+
         store = gr_recipe_store_get ();
 
         name = gtk_entry_get_text (GTK_ENTRY (page->name_entry));
+
+        if (name[0] == '\0') {
+                page->error_field = page->name_entry;
+                g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                             _("You need to provide a name for the recipe"));
+                goto error;
+        }
+
         cuisine = get_combo_value (GTK_COMBO_BOX (page->cuisine_combo));
         category = get_combo_value (GTK_COMBO_BOX (page->category_combo));
         season = get_combo_value (GTK_COMBO_BOX (page->season_combo));
@@ -1503,7 +1542,12 @@ gr_edit_page_save (GrEditPage *page)
         cook_time = get_combo_value (GTK_COMBO_BOX (page->cook_time_combo));
         spiciness = get_spiciness (page);
         serves = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (page->serves_spin));
-        ingredients = collect_ingredients (page);
+        if (!collect_ingredients (page, &page->error_field, &ingredients)) {
+                g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                             _("Some ingredients need correction"));
+                goto error;
+        }
+
         description = get_text_view_text (GTK_TEXT_VIEW (page->description_field));
         instructions = get_text_view_text (GTK_TEXT_VIEW (page->instructions_field));
         diets = (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->gluten_free_check)) ? GR_DIET_GLUTEN_FREE : 0) |
@@ -1583,6 +1627,7 @@ gr_edit_page_save (GrEditPage *page)
 
         gr_image_viewer_revert_changes (GR_IMAGE_VIEWER (page->images));
 
+error:
         gtk_label_set_label (GTK_LABEL (page->error_label), error->message);
         gtk_revealer_set_reveal_child (GTK_REVEALER (page->error_revealer), TRUE);
 
