@@ -27,6 +27,8 @@
 #include "gr-ingredients-viewer.h"
 #include "gr-ingredient.h"
 #include "gr-unit.h"
+#include "gr-utils.h"
+#include "gr-number.h"
 #include "gr-recipe-store.h"
 
 
@@ -41,9 +43,10 @@ struct _GrIngredientsViewerRow
         GtkWidget *ingredient_stack;
         GtkWidget *ingredient_label;
         GtkWidget *ingredient_entry;
-        GtkWidget *ebox;
+        GtkWidget *drag_handle;
         GtkWidget *unit_event_box;
         GtkWidget *ingredient_event_box;
+        GtkWidget *unit_help_popover;
 
         char *amount;
         char *unit;
@@ -52,6 +55,8 @@ struct _GrIngredientsViewerRow
         gboolean editable;
         gboolean active;
 
+        gboolean unit_error;
+
         GtkSizeGroup *group;
 };
 
@@ -141,9 +146,28 @@ update_unit (GrIngredientsViewerRow *row)
         space = amount[0] ? " " : "";
         unit = row->unit ? row->unit : "";
         tmp = g_strdup_printf ("%s%s%s", amount, space, unit);
-        gtk_label_set_label (GTK_LABEL (row->unit_label), tmp);
+        if (tmp[0] == '\0') {
+                gtk_style_context_add_class (gtk_widget_get_style_context (row->unit_label), "dim-label");
+                gtk_label_set_label (GTK_LABEL (row->unit_label), _("Amount…"));
+        }
+        else {
+                gtk_style_context_remove_class (gtk_widget_get_style_context (row->unit_label), "dim-label");
+                gtk_label_set_label (GTK_LABEL (row->unit_label), tmp);
+        }
 }
 
+static void
+update_ingredient (GrIngredientsViewerRow *row)
+{
+      if (row->ingredient[0] == '\0') {
+                gtk_style_context_add_class (gtk_widget_get_style_context (row->ingredient_label), "dim-label");
+                gtk_label_set_label (GTK_LABEL (row->ingredient_label), _("Ingredient…"));
+      }
+      else {
+                gtk_style_context_remove_class (gtk_widget_get_style_context (row->ingredient_label), "dim-label");
+                gtk_label_set_label (GTK_LABEL (row->ingredient_label), row->ingredient);
+      }
+}
 
 static void
 gr_ingredients_viewer_row_set_amount (GrIngredientsViewerRow *row,
@@ -169,7 +193,7 @@ gr_ingredients_viewer_row_set_ingredient (GrIngredientsViewerRow *row,
 {
         g_free (row->ingredient);
         row->ingredient = g_strdup (ingredient);
-        gtk_label_set_label (GTK_LABEL (row->ingredient_label), ingredient);
+        update_ingredient (row);
 }
 
 static void
@@ -193,9 +217,9 @@ gr_ingredients_viewer_row_set_editable (GrIngredientsViewerRow *row,
         setup_editable_row (row);
 }
 
-static void save_row (GrIngredientsViewerRow *row);
-static void save_unit (GrIngredientsViewerRow *row);
-static void save_ingredient (GrIngredientsViewerRow *row);
+static gboolean save_unit       (GrIngredientsViewerRow *row);
+static gboolean save_ingredient (GrIngredientsViewerRow *row);
+static void     save_row        (GrIngredientsViewerRow *row);
 
 static void
 gr_ingredients_viewer_row_set_active (GrIngredientsViewerRow *row,
@@ -253,17 +277,16 @@ emit_delete (GrIngredientsViewerRow *row)
         g_signal_emit (row, signals[DELETE], 0);
 }
 
-static void
-drag_handle_clicked (GrIngredientsViewerRow *row)
-{
-}
-
 static void
 edit_ingredient (GrIngredientsViewerRow *row)
 {
-        GrIngredientsViewer *viewer = GR_INGREDIENTS_VIEWER (gtk_widget_get_ancestor (GTK_WIDGET (row), GR_TYPE_INGREDIENTS_VIEWER));
-        save_unit (row);
+        GrIngredientsViewer *viewer;
+
+        viewer = GR_INGREDIENTS_VIEWER (gtk_widget_get_ancestor (GTK_WIDGET (row), GR_TYPE_INGREDIENTS_VIEWER));
+
         if (row->editable) {
+                save_unit (row);
+
                 set_active_row (viewer, GTK_WIDGET (row));
                 gtk_entry_set_text (GTK_ENTRY (row->ingredient_entry), row->ingredient);
                 gtk_stack_set_visible_child_name (GTK_STACK (row->ingredient_stack), "ingredient_entry");
@@ -272,6 +295,17 @@ edit_ingredient (GrIngredientsViewerRow *row)
         }
 }
 
+static gboolean
+show_help (gpointer data)
+{
+        GrIngredientsViewerRow *row = data;
+
+        if (row->unit_error)
+                gtk_popover_popup (GTK_POPOVER (row->unit_help_popover));
+
+        return G_SOURCE_REMOVE;
+}
+
 static void
 edit_unit (GrIngredientsViewerRow *row)
 {
@@ -279,11 +313,15 @@ edit_unit (GrIngredientsViewerRow *row)
         const char *amount;
         const char *space;
         const char *unit;
+        GrIngredientsViewer *viewer;
+
         amount = row->amount ? row->amount : "";
         space = amount[0] ? " " : "";
         unit = row->unit ? row->unit : "";
         tmp = g_strdup_printf ("%s%s%s", amount, space, unit);
-        GrIngredientsViewer *viewer = GR_INGREDIENTS_VIEWER (gtk_widget_get_ancestor (GTK_WIDGET (row), GR_TYPE_INGREDIENTS_VIEWER));
+
+        viewer = GR_INGREDIENTS_VIEWER (gtk_widget_get_ancestor (GTK_WIDGET (row), GR_TYPE_INGREDIENTS_VIEWER));
+
         save_ingredient (row);
 
         if (row->editable) {
@@ -291,6 +329,7 @@ edit_unit (GrIngredientsViewerRow *row)
                 gtk_entry_set_text (GTK_ENTRY (row->unit_entry), tmp);
                 gtk_stack_set_visible_child_name (GTK_STACK (row->unit_stack), "unit_entry");
                 gtk_widget_grab_focus (row->unit_entry);
+                show_help (row);
                 g_signal_emit (row, signals[EDIT], 0);
         }
 }
@@ -304,48 +343,88 @@ edit_unit_or_focus_out (GrIngredientsViewerRow *row)
                 save_row (row);
 }
 
-static void
+static gboolean
 parse_unit (const char  *text,
             char       **amount,
             char       **unit)
 {
-        g_autofree char *tmp = NULL;
-        g_autofree char **strv = NULL;
-
-        tmp = g_strstrip (g_strdup (text));
-        strv = g_strsplit (tmp, " ", 2);
+        char *tmp;
+        const char *str;
+        GrNumber number;
 
         g_clear_pointer (amount, g_free);
         g_clear_pointer (unit, g_free);
 
-        *amount = strv[0];
-        if (g_strv_length (strv) > 1)
-                *unit = strv[1];
+        tmp = (char *)text;
+        skip_whitespace (&tmp);
+        str = tmp;
+        if (!gr_number_parse (&number, &tmp, NULL)) {
+                *unit = g_strdup (str);
+                return FALSE;
+        }
+
+        *amount = g_strndup (str, tmp - str);
+        skip_whitespace (&tmp);
+        if (tmp)
+                *unit = g_strdup (tmp);
+
+        return TRUE;
 }
 
 static void
+set_unit_error (GrIngredientsViewerRow *row,
+                gboolean                error)
+{
+        row->unit_error = error;
+
+        if (error) {
+                gtk_style_context_add_class (gtk_widget_get_style_context (row->unit_entry), "error");
+                gtk_style_context_add_class (gtk_widget_get_style_context (row->unit_label), "error");
+        }
+        else {
+                gtk_style_context_remove_class (gtk_widget_get_style_context (row->unit_entry), "error");
+                gtk_style_context_remove_class (gtk_widget_get_style_context (row->unit_label), "error");
+                gtk_popover_popdown (GTK_POPOVER (row->unit_help_popover));
+        }
+}
+
+static void
+unit_text_changed (GrIngredientsViewerRow *row)
+{
+        set_unit_error (row, FALSE);
+}
+
+static gboolean
 save_unit (GrIngredientsViewerRow *row)
 {
         GtkWidget *visible;
 
         visible = gtk_stack_get_visible_child (GTK_STACK (row->unit_stack));
         if (visible == row->unit_entry) {
-                parse_unit (gtk_entry_get_text (GTK_ENTRY (row->unit_entry)), &row->amount, &row->unit);
+                if (!parse_unit (gtk_entry_get_text (GTK_ENTRY (row->unit_entry)), &row->amount, &row->unit)) {
+                        set_unit_error (row, TRUE);
+                }
+
                 update_unit (row);
                 gtk_stack_set_visible_child (GTK_STACK (row->unit_stack), row->unit_event_box);
         }
+
+        return GDK_EVENT_PROPAGATE;
 }
 
-static void
+static gboolean
 save_ingredient (GrIngredientsViewerRow *row)
 {
         GtkWidget *visible;
+
         visible = gtk_stack_get_visible_child (GTK_STACK (row->ingredient_stack));
         if (visible == row->ingredient_entry) {
                 row->ingredient = g_strdup (gtk_entry_get_text (GTK_ENTRY (row->ingredient_entry)));
-                gtk_label_set_label (GTK_LABEL (row->ingredient_label), row->ingredient);
+                update_ingredient (row);
                 gtk_stack_set_visible_child (GTK_STACK (row->ingredient_stack), row->ingredient_event_box);
         }
+
+        return GDK_EVENT_PROPAGATE;
 }
 
 static void
@@ -368,6 +447,32 @@ entry_key_press (GrIngredientsViewerRow *row,
         return GDK_EVENT_PROPAGATE;
 }
 
+static gboolean
+drag_key_press (GrIngredientsViewerRow *source,
+                GdkEventKey            *event)
+{
+        GtkWidget *list;
+        GtkWidget *target;
+        int index;
+
+        list = gtk_widget_get_parent (GTK_WIDGET (source));
+        index = gtk_list_box_row_get_index (GTK_LIST_BOX_ROW (source));
+
+        if ((event->state & GDK_MOD1_MASK) != 0 && event->keyval == GDK_KEY_Up)
+                target = GTK_WIDGET (gtk_list_box_get_row_at_index (GTK_LIST_BOX (list), index - 1));
+        else if ((event->state & GDK_MOD1_MASK) != 0 && event->keyval == GDK_KEY_Down)
+                target = GTK_WIDGET (gtk_list_box_get_row_at_index (GTK_LIST_BOX (list), index + 1));
+        else
+                return GDK_EVENT_PROPAGATE;
+
+        if (target) {
+                g_signal_emit (source, signals[MOVE], 0, target);
+                gtk_widget_grab_focus (GTK_WIDGET (source));
+        }
+
+        return GDK_EVENT_STOP;
+}
+
 static void
 gr_ingredients_viewer_row_class_init (GrIngredientsViewerRowClass *klass)
 {
@@ -424,7 +529,7 @@ gr_ingredients_viewer_row_class_init (GrIngredientsViewerRowClass *klass)
                                       NULL, NULL,
                                       NULL,
                                       G_TYPE_NONE, 1, GR_TYPE_INGREDIENTS_VIEWER_ROW);
-                signals[EDIT] = g_signal_new ("edit",
+        signals[EDIT] = g_signal_new ("edit",
                                       G_TYPE_FROM_CLASS (object_class),
                                       G_SIGNAL_RUN_FIRST,
                                       0,
@@ -441,17 +546,20 @@ gr_ingredients_viewer_row_class_init (GrIngredientsViewerRowClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrIngredientsViewerRow, ingredient_label);
         gtk_widget_class_bind_template_child (widget_class, GrIngredientsViewerRow, ingredient_entry);
         gtk_widget_class_bind_template_child (widget_class, GrIngredientsViewerRow, buttons_stack);
-        gtk_widget_class_bind_template_child (widget_class, GrIngredientsViewerRow, ebox);
+        gtk_widget_class_bind_template_child (widget_class, GrIngredientsViewerRow, drag_handle);
         gtk_widget_class_bind_template_child (widget_class, GrIngredientsViewerRow, unit_event_box);
         gtk_widget_class_bind_template_child (widget_class, GrIngredientsViewerRow, ingredient_event_box);
+        gtk_widget_class_bind_template_child (widget_class, GrIngredientsViewerRow, unit_help_popover);
 
         gtk_widget_class_bind_template_callback (widget_class, emit_delete);
-        gtk_widget_class_bind_template_callback (widget_class, drag_handle_clicked);
         gtk_widget_class_bind_template_callback (widget_class, edit_unit);
+        gtk_widget_class_bind_template_callback (widget_class, save_unit);
         gtk_widget_class_bind_template_callback (widget_class, edit_unit_or_focus_out);
         gtk_widget_class_bind_template_callback (widget_class, edit_ingredient);
-        gtk_widget_class_bind_template_callback (widget_class, save_row);
+        gtk_widget_class_bind_template_callback (widget_class, save_ingredient);
         gtk_widget_class_bind_template_callback (widget_class, entry_key_press);
+        gtk_widget_class_bind_template_callback (widget_class, drag_key_press);
+        gtk_widget_class_bind_template_callback (widget_class, unit_text_changed);
 }
 
 static GtkTargetEntry entries[] = {
@@ -670,9 +778,9 @@ setup_editable_row (GrIngredientsViewerRow *self)
                 g_autoptr(GtkTreeModel) ingredients_model = NULL;
                 g_autoptr(GtkTreeModel) units_model = NULL;
 
-                gtk_drag_source_set (self->ebox, GDK_BUTTON1_MASK, entries, 1, GDK_ACTION_MOVE);
-                g_signal_connect (self->ebox, "drag-begin", G_CALLBACK (drag_begin), NULL);
-                g_signal_connect (self->ebox, "drag-data-get", G_CALLBACK (drag_data_get), NULL);
+                gtk_drag_source_set (self->drag_handle, GDK_BUTTON1_MASK, entries, 1, GDK_ACTION_MOVE);
+                g_signal_connect (self->drag_handle, "drag-begin", G_CALLBACK (drag_begin), NULL);
+                g_signal_connect (self->drag_handle, "drag-data-get", G_CALLBACK (drag_data_get), NULL);
 
                 gtk_drag_dest_set (GTK_WIDGET (self), GTK_DEST_DEFAULT_ALL, entries, 1, GDK_ACTION_MOVE);
                 g_signal_connect (self, "drag-data-received", G_CALLBACK (drag_data_received), NULL);
@@ -690,9 +798,9 @@ setup_editable_row (GrIngredientsViewerRow *self)
                 gtk_entry_set_completion (GTK_ENTRY (self->unit_entry), completion);
         }
         else {
-                gtk_drag_source_unset (self->ebox);
-                g_signal_handlers_disconnect_by_func (self->ebox, drag_begin, NULL);
-                g_signal_handlers_disconnect_by_func (self->ebox, drag_data_get, NULL);
+                gtk_drag_source_unset (self->drag_handle);
+                g_signal_handlers_disconnect_by_func (self->drag_handle, drag_begin, NULL);
+                g_signal_handlers_disconnect_by_func (self->drag_handle, drag_data_get, NULL);
 
                 gtk_drag_dest_unset (GTK_WIDGET (self));
                 g_signal_handlers_disconnect_by_func (self, drag_data_received, NULL);
@@ -701,3 +809,15 @@ setup_editable_row (GrIngredientsViewerRow *self)
                 gtk_entry_set_completion (GTK_ENTRY (self->unit_entry), NULL);
         }
 }
+
+GtkWidget *
+gr_ingredients_viewer_row_has_error (GrIngredientsViewerRow *row)
+{
+        if (row->amount == NULL || row->amount[0] == '\0')
+                return gtk_widget_get_parent (row->unit_label);
+
+        if (row->ingredient == NULL || row->ingredient[0] == '\0')
+                return gtk_widget_get_parent (row->ingredient_label);
+
+        return NULL;
+}
