@@ -27,10 +27,11 @@
 #include "gr-recipe-store.h"
 #include "gr-recipe.h"
 #include "gr-recipe-tile.h"
-#include "gr-app.h"
 #include "gr-utils.h"
 #include "gr-season.h"
 #include "gr-category-tile.h"
+#include "gr-image.h"
+#include "gr-app.h"
 
 struct _GrListPage
 {
@@ -43,6 +44,8 @@ struct _GrListPage
         gboolean new;
         char *season;
         GList *recipes;
+        GrImage *ri;
+        GCancellable *cancellable;
 
         GtkWidget *top_box;
         GtkWidget *list_stack;
@@ -65,6 +68,7 @@ G_DEFINE_TYPE (GrListPage, gr_list_page, GTK_TYPE_BOX)
 
 static void connect_store_signals (GrListPage *page);
 
+/* keep this function in sync with gr_list_page_repopulate */
 static void
 clear_data (GrListPage *self)
 {
@@ -73,6 +77,9 @@ clear_data (GrListPage *self)
                                                 gr_chef_get_id (self->chef));
         }
 
+        g_cancellable_cancel (self->cancellable);
+        g_clear_object (&self->cancellable);
+        g_clear_object (&self->ri);
         g_clear_object (&self->chef);
         self->diet = 0;
         self->favorites = FALSE;
@@ -88,6 +95,9 @@ list_page_finalize (GObject *object)
 {
         GrListPage *self = GR_LIST_PAGE (object);
 
+        g_cancellable_cancel (self->cancellable);
+        g_clear_object (&self->cancellable);
+        g_clear_object (&self->ri);
         g_clear_object (&self->chef);
         g_clear_pointer (&self->season, g_free);
         g_list_free_full (self->recipes, g_object_unref);
@@ -317,14 +327,14 @@ gr_list_page_populate_from_chef (GrListPage *self,
         description = gr_chef_get_translated_description (chef) ? gr_chef_get_translated_description (chef) : "";
         path = gr_chef_get_image (chef);
 
+        gtk_image_clear (GTK_IMAGE (self->chef_image));
         if (path && path[0]) {
-                g_autoptr(GdkPixbuf) pixbuf = NULL;
+                self->ri = gr_image_new (gr_app_get_soup_session (GR_APP (g_application_get_default ())),
+                                         gr_chef_get_id (chef),
+                                         path);
+                self->cancellable = g_cancellable_new ();
 
-                pixbuf = load_pixbuf_fill_size (path, 64, 64);
-                gtk_image_set_from_pixbuf (GTK_IMAGE (self->chef_image), pixbuf);
-        }
-        else {
-                gtk_image_clear (GTK_IMAGE (self->chef_image));
+                gr_image_load (self->ri, 64, 64, FALSE, self->cancellable, gr_image_set_pixbuf, self->chef_image);
         }
 
         gtk_widget_show (self->chef_grid);
@@ -338,7 +348,7 @@ gr_list_page_populate_from_chef (GrListPage *self,
         gtk_label_set_label (GTK_LABEL (self->heading), tmp);
         g_free (tmp);
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         container_remove_all (GTK_CONTAINER (self->flow_box));
         tmp = g_strdup_printf (_("No recipes by chef %s found"), name);
@@ -461,7 +471,7 @@ gr_list_page_populate_from_list (GrListPage *self,
         GList *l;
         gboolean empty;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         recipes = g_list_copy_deep (recipes, (GCopyFunc)g_object_ref, NULL);
         clear_data (self);
@@ -495,6 +505,7 @@ gr_list_page_populate_from_list (GrListPage *self,
                                           empty ? "empty" : "list");
 }
 
+/* keep function this in sync with clear_data */
 void
 gr_list_page_repopulate (GrListPage *page)
 {
@@ -510,6 +521,8 @@ gr_list_page_repopulate (GrListPage *page)
                 gr_list_page_populate_from_list (page, page->recipes);
         else if (page->all)
                 gr_list_page_populate_from_all (page);
+        else if (page->new)
+                gr_list_page_populate_from_new (page);
 }
 
 static void
@@ -524,7 +537,7 @@ connect_store_signals (GrListPage *page)
 {
         GrRecipeStore *store;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
 
         g_signal_connect_swapped (store, "recipe-added", G_CALLBACK (repopulate), page);
         g_signal_connect_swapped (store, "recipe-removed", G_CALLBACK (repopulate), page);
