@@ -25,9 +25,10 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "gr-recipe-tile.h"
+#include "gr-recipe-store.h"
 #include "gr-window.h"
 #include "gr-utils.h"
-#include "gr-images.h"
+#include "gr-image.h"
 
 
 struct _GrRecipeTile
@@ -42,6 +43,8 @@ struct _GrRecipeTile
         GtkWidget *author;
         GtkWidget *image;
         GtkWidget *box;
+
+        GCancellable *cancellable;
 };
 
 G_DEFINE_TYPE (GrRecipeTile, gr_recipe_tile, GTK_TYPE_BUTTON)
@@ -61,7 +64,10 @@ recipe_tile_set_recipe (GrRecipeTile *tile,
 {
         GrRecipeStore *store;
 
-        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        store = gr_recipe_store_get ();
+
+        g_cancellable_cancel (tile->cancellable);
+        g_clear_object (&tile->cancellable);
 
         g_set_object (&tile->recipe, recipe);
 
@@ -70,7 +76,7 @@ recipe_tile_set_recipe (GrRecipeTile *tile,
                 const char *author;
                 g_autoptr(GrChef) chef = NULL;
                 g_autofree char *tmp = NULL;
-                g_autoptr(GArray) images = NULL;
+                GPtrArray *images;
 
                 name = gr_recipe_get_translated_name (recipe);
                 author = gr_recipe_get_author (recipe);
@@ -80,24 +86,24 @@ recipe_tile_set_recipe (GrRecipeTile *tile,
                 tmp = g_strdup_printf (_("by %s"), chef ? gr_chef_get_name (chef) : _("Anonymous"));
                 gtk_label_set_label (GTK_LABEL (tile->author), tmp);
 
-                g_object_get (recipe, "images", &images, NULL);
+                images = gr_recipe_get_images (recipe);
                 if (images->len > 0) {
                         int index;
                         GrImage *ri;
-                        g_autoptr(GdkPixbuf) pixbuf = NULL;
-                        int width;
-                        int height;
 
-                        width = tile->wide ? 538 : 258;
-                        height = 200;
+                        tile->cancellable = g_cancellable_new ();
 
                         index = gr_recipe_get_default_image (recipe);
                         if (index < 0 || index >= images->len)
                                 index = 0;
 
-                        ri = &g_array_index (images, GrImage, index);
-                        pixbuf = load_pixbuf_fill_size (ri->path, width, height);
-                        gtk_image_set_from_pixbuf (GTK_IMAGE (tile->image), pixbuf);
+                        ri = g_ptr_array_index (images, index);
+
+                        gr_image_load (ri,
+                                       tile->wide ? 538 : 258, 200, FALSE,
+                                       tile->cancellable,
+                                       gr_image_set_pixbuf,
+                                       tile->image);
                 }
         }
 }
@@ -107,6 +113,8 @@ recipe_tile_finalize (GObject *object)
 {
         GrRecipeTile *tile = GR_RECIPE_TILE (object);
 
+        g_cancellable_cancel (tile->cancellable);
+        g_clear_object (&tile->cancellable);
         g_clear_object (&tile->recipe);
 
         G_OBJECT_CLASS (gr_recipe_tile_parent_class)->finalize (object);
