@@ -852,8 +852,10 @@ done_shopping (GrWindow *window)
         gr_recipe_store_clear_shopping_list (store);
 }
 
-static
-void make_save_sensitive (GrEditPage *edit_page, GParamSpec *pspec, gpointer data)
+static void
+make_save_sensitive (GrEditPage *edit_page,
+                     GParamSpec *pspec,
+                     gpointer    data)
 {
         GrWindow *window = GR_WINDOW (data);
         gboolean unsaved;
@@ -1206,14 +1208,13 @@ gr_window_save_all (GrWindow *window)
 
 void
 gr_window_show_diet (GrWindow   *window,
-                     const char *title,
                      GrDiets     diet)
 {
         save_back_entry (window);
 
         gr_list_page_populate_from_diet (GR_LIST_PAGE (window->list_page), diet);
 
-        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), title);
+        gtk_header_bar_set_title (GTK_HEADER_BAR (window->header), gr_diet_get_label (diet));
 
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_start_stack), "back");
         gtk_stack_set_visible_child_name (GTK_STACK (window->header_title_stack), "title");
